#include <iostream>
using namespace std;
/**
* The classes that implement a concrete strategy should inherit this.
* The Calculator class uses this to use a concrete strategy.
*/
class Strategy {
  public:
	  /**
	  * Method whose implementation varies depending on the strategy adopted.
	  */
	  virtual int execute(int a, int b) = 0;
};
/**
* A concrete Strategy that implements a addition operation
*/
class Add : public Strategy {
  public: 
	  int execute(int a, int b){
		  cout << "Called Add's execute()" << endl;
		  return a + b;
	  }
};
/**
* A concrete Strategy that implements a subtraction operation
*/
class Subtract : public Strategy {
  public: 
	  int execute(int a, int b){
		  cout << "Called Subtract's execute()" << endl;
		  return a - b;
	  }
};
/**
* A concrete Strategy that implements a multiplication operation
*/
class Multiply : public Strategy {
  public: 
	  int execute(int a, int b){
		  cout << "Called Multiply's execute()" << endl;
		  return a * b;
	  }
};
/** Class configured with a ConcreteStrategy object and maintains
*   a reference to a Strategy object. Depending on which strategy is
*   plugged in, it will execute a different operation.
*/
class Calculator {
  private: 
	  Strategy *strategy; 
  
  public: 
	  Calculator(){};
	  /**
	  * Plugs in a specific strategy to be used
	  */
	  Calculator(Strategy *initStrategy){
		this->strategy = initStrategy;
	  }
	  void setStrategy(Strategy *newStrategy){
		this->strategy = newStrategy;
	  }
	  /**
	  * Method that executes a different strategy depending on what strategy was
	  * plugged in upon instantiation.
	  */
	  int executeStrategy(int a, int b){
		  return this->strategy->execute(a, b);
	  }
};

int main(){
	Calculator calculator(new Add());
	int resultA = calculator.executeStrategy(3, 4);
	calculator.setStrategy(new Subtract());
	int resultB = calculator.executeStrategy(3, 4);
	calculator.setStrategy(new Multiply());
	int resultC = calculator.executeStrategy(3, 4);

	cout << "Result of addition strategy: " << resultA << endl;
	cout << "Result of subtraction strategy: " << resultB << endl;
	cout << "Result of multiplication strategy: " << resultC << endl;

	system("PAUSE");
	return EXIT_SUCCESS;
}